@@ -4,10 +4,11 @@ from __future__ import annotations
 
 import html
 import os
+import os.path
 import re
 import time
-from os import path
-from typing import TYPE_CHECKING, Any, NamedTuple
+from pathlib import Path
+from typing import TYPE_CHECKING, NamedTuple
 from urllib.parse import quote
 from zipfile import ZIP_DEFLATED, ZIP_STORED, ZipFile
 
@@ -19,11 +20,14 @@ from sphinx.builders.html import StandaloneHTMLBuilder
 from sphinx.builders.html._build_info import BuildInfo
 from sphinx.locale import __
 from sphinx.util import logging
+from sphinx.util._pathlib import _StrPath
 from sphinx.util.display import status_iterator
 from sphinx.util.fileutil import copy_asset_file
 from sphinx.util.osutil import copyfile, ensuredir, relpath
 
 if TYPE_CHECKING:
+    from typing import Any
+
     from docutils.nodes import Element, Node
 
 try:
@@ -110,8 +114,8 @@ class NavPoint(NamedTuple):
 
 def sphinx_smarty_pants(t: str, language: str = 'en') -> str:
     t = t.replace('&quot;', '"')
-    t = smartquotes.educateDashesOldSchool(t)  # type: ignore[no-untyped-call]
-    t = smartquotes.educateQuotes(t, language)  # type: ignore[no-untyped-call]
+    t = smartquotes.educateDashesOldSchool(t)
+    t = smartquotes.educateQuotes(t, language)
     t = t.replace('"', '&quot;')
     return t
 
@@ -123,8 +127,7 @@ ssp = sphinx_smarty_pants
 
 
 class EpubBuilder(StandaloneHTMLBuilder):
-    """
-    Builder that outputs epub files.
+    """Builder that outputs epub files.
 
     It creates the metainfo files container.opf, toc.ncx, mimetype, and
     META-INF/container.xml.  Afterwards, all necessary files are zipped to an
@@ -156,7 +159,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
     guide_titles = GUIDE_TITLES
     media_types = MEDIA_TYPES
     refuri_re = REFURI_RE
-    template_dir = ''
+    template_dir: _StrPath = _StrPath()
     doctype = ''
 
     def init(self) -> None:
@@ -226,18 +229,20 @@ class EpubBuilder(StandaloneHTMLBuilder):
                 appeared.add(node['refuri'])
 
     def get_toc(self) -> None:
-        """Get the total table of contents, containing the root_doc
-        and pre and post files not managed by sphinx.
+        """Get the total table of contents, containing the master_doc
+        and pre and post files not managed by Sphinx.
         """
         doctree = self.env.get_and_resolve_doctree(
-            self.config.root_doc, self, prune_toctrees=False, includehidden=True
+            self.config.master_doc,
+            self,
+            tags=self.tags,
+            prune_toctrees=False,
+            includehidden=True,
         )
         self.refnodes = self.get_refnodes(doctree, [])
-        master_dir = path.dirname(self.config.root_doc)
-        if master_dir:
-            master_dir += '/'  # XXX or os.sep?
-            for item in self.refnodes:
-                item['refuri'] = master_dir + item['refuri']
+        master_dir = Path(self.config.master_doc).parent
+        for item in self.refnodes:
+            item['refuri'] = str(master_dir / item['refuri'])
         self.toc_add_files(self.refnodes)
 
     def toc_add_files(self, refnodes: list[dict[str, Any]]) -> None:
@@ -278,16 +283,6 @@ class EpubBuilder(StandaloneHTMLBuilder):
         Some readers crash because they interpret the part as a
         transport protocol specification.
         """
-
-        def update_node_id(node: Element) -> None:
-            """Update IDs of given *node*."""
-            new_ids: list[str] = []
-            for node_id in node['ids']:
-                new_id = self.fix_fragment('', node_id)
-                if new_id not in new_ids:
-                    new_ids.append(new_id)
-            node['ids'] = new_ids
-
         for reference in tree.findall(nodes.reference):
             if 'refuri' in reference:
                 m = self.refuri_re.match(reference['refuri'])
@@ -297,66 +292,75 @@ class EpubBuilder(StandaloneHTMLBuilder):
                 reference['refid'] = self.fix_fragment('', reference['refid'])
 
         for target in tree.findall(nodes.target):
-            update_node_id(target)
+            self._update_node_id(target)
 
-            next_node: Node = target.next_node(ascend=True)
+            next_node: Node | None = target.next_node(ascend=True)
             if isinstance(next_node, nodes.Element):
-                update_node_id(next_node)
+                self._update_node_id(next_node)
 
         for desc_signature in tree.findall(addnodes.desc_signature):
-            update_node_id(desc_signature)
+            self._update_node_id(desc_signature)
+
+    def _update_node_id(self, node: Element, /) -> None:
+        """Update IDs of given *node*."""
+        new_ids: list[str] = []
+        for node_id in node['ids']:
+            new_id = self.fix_fragment('', node_id)
+            if new_id not in new_ids:
+                new_ids.append(new_id)
+        node['ids'] = new_ids
+
+    @staticmethod
+    def _make_footnote_ref(doc: nodes.document, label: str) -> nodes.footnote_reference:
+        """Create a footnote_reference node with children"""
+        footnote_ref = nodes.footnote_reference('[#]_')
+        footnote_ref.append(nodes.Text(label))
+        doc.note_autofootnote_ref(footnote_ref)
+        return footnote_ref
+
+    @staticmethod
+    def _make_footnote(doc: nodes.document, label: str, uri: str) -> nodes.footnote:
+        """Create a footnote node with children"""
+        footnote = nodes.footnote(uri)
+        para = nodes.paragraph()
+        para.append(nodes.Text(uri))
+        footnote.append(para)
+        footnote.insert(0, nodes.label('', label))
+        doc.note_autofootnote(footnote)
+        return footnote
+
+    @staticmethod
+    def _footnote_spot(tree: nodes.document) -> tuple[Element, int]:
+        """Find or create a spot to place footnotes.
+
+        The function returns the tuple (parent, index).
+        """
+        # The code uses the following heuristic:
+        # a) place them after the last existing footnote
+        # b) place them after an (empty) Footnotes rubric
+        # c) create an empty Footnotes rubric at the end of the document
+        fns = list(tree.findall(nodes.footnote))
+        if fns:
+            fn = fns[-1]
+            return fn.parent, fn.parent.index(fn) + 1
+        for node in tree.findall(nodes.rubric):
+            if len(node) == 1 and node.astext() == FOOTNOTES_RUBRIC_NAME:
+                return node.parent, node.parent.index(node) + 1
+        doc = next(tree.findall(nodes.document))
+        rub = nodes.rubric()
+        rub.append(nodes.Text(FOOTNOTES_RUBRIC_NAME))
+        doc.append(rub)
+        return doc, doc.index(rub) + 1
 
     def add_visible_links(
         self, tree: nodes.document, show_urls: str = 'inline'
     ) -> None:
         """Add visible link targets for external links"""
-
-        def make_footnote_ref(
-            doc: nodes.document, label: str
-        ) -> nodes.footnote_reference:
-            """Create a footnote_reference node with children"""
-            footnote_ref = nodes.footnote_reference('[#]_')
-            footnote_ref.append(nodes.Text(label))
-            doc.note_autofootnote_ref(footnote_ref)
-            return footnote_ref
-
-        def make_footnote(doc: nodes.document, label: str, uri: str) -> nodes.footnote:
-            """Create a footnote node with children"""
-            footnote = nodes.footnote(uri)
-            para = nodes.paragraph()
-            para.append(nodes.Text(uri))
-            footnote.append(para)
-            footnote.insert(0, nodes.label('', label))
-            doc.note_autofootnote(footnote)
-            return footnote
-
-        def footnote_spot(tree: nodes.document) -> tuple[Element, int]:
-            """Find or create a spot to place footnotes.
-
-            The function returns the tuple (parent, index).
-            """
-            # The code uses the following heuristic:
-            # a) place them after the last existing footnote
-            # b) place them after an (empty) Footnotes rubric
-            # c) create an empty Footnotes rubric at the end of the document
-            fns = list(tree.findall(nodes.footnote))
-            if fns:
-                fn = fns[-1]
-                return fn.parent, fn.parent.index(fn) + 1
-            for node in tree.findall(nodes.rubric):
-                if len(node) == 1 and node.astext() == FOOTNOTES_RUBRIC_NAME:
-                    return node.parent, node.parent.index(node) + 1
-            doc = next(tree.findall(nodes.document))
-            rub = nodes.rubric()
-            rub.append(nodes.Text(FOOTNOTES_RUBRIC_NAME))
-            doc.append(rub)
-            return doc, doc.index(rub) + 1
-
         if show_urls == 'no':
             return
         if show_urls == 'footnote':
             doc = next(tree.findall(nodes.document))
-            fn_spot, fn_idx = footnote_spot(tree)
+            fn_spot, fn_idx = self._footnote_spot(tree)
             nr = 1
         for node in list(tree.findall(nodes.reference)):
             uri = node.get('refuri', '')
@@ -370,9 +374,9 @@ class EpubBuilder(StandaloneHTMLBuilder):
                 elif show_urls == 'footnote':
                     label = FOOTNOTE_LABEL_TEMPLATE % nr
                     nr += 1
-                    footnote_ref = make_footnote_ref(doc, label)
+                    footnote_ref = self._make_footnote_ref(doc, label)
                     node.parent.insert(idx, footnote_ref)
-                    footnote = make_footnote(doc, label, uri)
+                    footnote = self._make_footnote(doc, label, uri)
                     fn_spot.insert(fn_idx, footnote)
                     footnote_ref['refid'] = footnote['ids'][0]
                     footnote.add_backref(footnote_ref['ids'][0])
@@ -407,7 +411,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
 
     def is_vector_graphics(self, filename: str) -> bool:
         """Does the filename extension indicate a vector graphic format?"""
-        ext = path.splitext(filename)[-1]
+        ext = os.path.splitext(filename)[-1]
         return ext in VECTOR_GRAPHICS_EXTENSIONS
 
     def copy_image_files_pil(self) -> None:
@@ -415,22 +419,22 @@ class EpubBuilder(StandaloneHTMLBuilder):
         The method tries to read and write the files with Pillow, converting
         the format and resizing the image if necessary/possible.
         """
-        ensuredir(path.join(self.outdir, self.imagedir))
+        ensuredir(self.outdir / self.imagedir)
         for src in status_iterator(
             self.images,
             __('copying images... '),
             'brown',
             len(self.images),
-            self.app.verbosity,
+            self.config.verbosity,
         ):
             dest = self.images[src]
             try:
-                img = Image.open(path.join(self.srcdir, src))
+                img = Image.open(self.srcdir / src)
             except OSError:
                 if not self.is_vector_graphics(src):
                     logger.warning(
                         __('cannot read image file %r: copying it instead'),
-                        path.join(self.srcdir, src),
+                        self.srcdir / src,
                     )
                 try:
                     copyfile(
@@ -441,7 +445,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
                 except OSError as err:
                     logger.warning(
                         __('cannot copy image file %r: %s'),
-                        path.join(self.srcdir, src),
+                        self.srcdir / src,
                         err,
                     )
                 continue
@@ -457,11 +461,11 @@ class EpubBuilder(StandaloneHTMLBuilder):
                     nh = round((height * nw) / width)
                     img = img.resize((nw, nh), Image.BICUBIC)
             try:
-                img.save(path.join(self.outdir, self.imagedir, dest))
+                img.save(self.outdir / self.imagedir / dest)
             except OSError as err:
                 logger.warning(
                     __('cannot write image file %r: %s'),
-                    path.join(self.srcdir, src),
+                    self.srcdir / src,
                     err,
                 )
 
@@ -487,7 +491,8 @@ class EpubBuilder(StandaloneHTMLBuilder):
         pagename: str,
         addctx: dict[str, Any],
         templatename: str = 'page.html',
-        outfilename: str | None = None,
+        *,
+        outfilename: Path | None = None,
         event_arg: Any = None,
     ) -> None:
         """Create a rendered page.
@@ -500,13 +505,15 @@ class EpubBuilder(StandaloneHTMLBuilder):
                 return
             self.fix_genindex(addctx['genindexentries'])
         addctx['doctype'] = self.doctype
-        super().handle_page(pagename, addctx, templatename, outfilename, event_arg)
+        super().handle_page(
+            pagename, addctx, templatename, outfilename=outfilename, event_arg=event_arg
+        )
 
     def build_mimetype(self) -> None:
         """Write the metainfo file mimetype."""
         logger.info(__('writing mimetype file...'))
         copyfile(
-            path.join(self.template_dir, 'mimetype'),
+            self.template_dir / 'mimetype',
             self.outdir / 'mimetype',
             force=True,
         )
@@ -517,7 +524,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
         outdir = self.outdir / 'META-INF'
         ensuredir(outdir)
         copyfile(
-            path.join(self.template_dir, 'container.xml'),
+            self.template_dir / 'container.xml',
             outdir / 'container.xml',
             force=True,
         )
@@ -531,19 +538,20 @@ class EpubBuilder(StandaloneHTMLBuilder):
         else:
             time_tuple = time.gmtime()
 
-        metadata: dict[str, Any] = {}
-        metadata['title'] = html.escape(self.config.epub_title)
-        metadata['author'] = html.escape(self.config.epub_author)
-        metadata['uid'] = html.escape(self.config.epub_uid)
-        metadata['lang'] = html.escape(self.config.epub_language)
-        metadata['publisher'] = html.escape(self.config.epub_publisher)
-        metadata['copyright'] = html.escape(self.config.epub_copyright)
-        metadata['scheme'] = html.escape(self.config.epub_scheme)
-        metadata['id'] = html.escape(self.config.epub_identifier)
-        metadata['date'] = html.escape(time.strftime('%Y-%m-%d', time_tuple))
-        metadata['manifest_items'] = []
-        metadata['spines'] = []
-        metadata['guides'] = []
+        metadata: dict[str, Any] = {
+            'title': html.escape(self.config.epub_title),
+            'author': html.escape(self.config.epub_author),
+            'uid': html.escape(self.config.epub_uid),
+            'lang': html.escape(self.config.epub_language),
+            'publisher': html.escape(self.config.epub_publisher),
+            'copyright': html.escape(self.config.epub_copyright),
+            'scheme': html.escape(self.config.epub_scheme),
+            'id': html.escape(self.config.epub_identifier),
+            'date': html.escape(time.strftime('%Y-%m-%d', time_tuple)),
+            'manifest_items': [],
+            'spines': [],
+            'guides': [],
+        }
         return metadata
 
     def build_content(self) -> None:
@@ -571,16 +579,17 @@ class EpubBuilder(StandaloneHTMLBuilder):
         if not self.use_index:
             self.ignored_files.append('genindex' + self.out_suffix)
         for root, dirs, files in os.walk(self.outdir):
+            root_path = Path(root)
             dirs.sort()
             for fn in sorted(files):
-                filename = relpath(path.join(root, fn), self.outdir)
+                filename = relpath(root_path / fn, self.outdir)
                 if filename in self.ignored_files:
                     continue
-                ext = path.splitext(filename)[-1]
+                ext = os.path.splitext(filename)[-1]
                 if ext not in self.media_types:
                     # we always have JS and potentially OpenSearch files, don't
                     # always warn about them
-                    if ext not in ('.js', '.xml'):
+                    if ext not in {'.js', '.xml'}:
                         logger.warning(
                             __('unknown mimetype for %s, ignoring'),
                             filename,
@@ -604,9 +613,10 @@ class EpubBuilder(StandaloneHTMLBuilder):
                 continue
             if refnode['refuri'] in self.ignored_files:
                 continue
-            spine = Spine(html.escape(self.make_id(refnode['refuri'])), True)
+            refuri = refnode['refuri'].replace(os.sep, '/')
+            spine = Spine(html.escape(self.make_id(refuri)), True)
             metadata['spines'].append(spine)
-            spinefiles.add(refnode['refuri'])
+            spinefiles.add(refuri)
         for info in self.domain_indices:
             spine = Spine(html.escape(self.make_id(info[0] + self.out_suffix)), True)
             metadata['spines'].append(spine)
@@ -631,7 +641,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
                 spine = Spine(html.escape(self.make_id(self.coverpage_name)), True)
                 metadata['spines'].insert(0, spine)
                 if self.coverpage_name not in self.files:
-                    ext = path.splitext(self.coverpage_name)[-1]
+                    ext = os.path.splitext(self.coverpage_name)[-1]
                     self.files.append(self.coverpage_name)
                     item = ManifestItem(
                         html.escape(self.coverpage_name),
@@ -640,7 +650,9 @@ class EpubBuilder(StandaloneHTMLBuilder):
                     )
                     metadata['manifest_items'].append(item)
                 ctx = {'image': html.escape(image), 'title': self.config.project}
-                self.handle_page(path.splitext(self.coverpage_name)[0], ctx, html_tmpl)
+                self.handle_page(
+                    os.path.splitext(self.coverpage_name)[0], ctx, html_tmpl
+                )
                 spinefiles.add(self.coverpage_name)
 
         auto_add_cover = True
@@ -676,7 +688,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
 
         # write the project file
         copy_asset_file(
-            path.join(self.template_dir, 'content.opf.jinja'),
+            self.template_dir / 'content.opf.jinja',
             self.outdir,
             context=metadata,
             force=True,
@@ -700,8 +712,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
         Subelements of a node are nested inside the navpoint.  For nested nodes
         the parent node is reinserted in the subnav.
         """
-        navstack: list[NavPoint] = []
-        navstack.append(NavPoint('dummy', 0, '', '', []))
+        navstack: list[NavPoint] = [NavPoint('dummy', 0, '', '', [])]
         level = 0
         lastnode = None
         for node in nodes:
@@ -735,7 +746,8 @@ class EpubBuilder(StandaloneHTMLBuilder):
                 navstack[-1].children.append(navpoint)
                 navstack.append(navpoint)
             else:
-                raise
+                msg = __('node has an invalid level')
+                raise ValueError(msg)
             lastnode = node
 
         return navstack[0].children
@@ -744,11 +756,12 @@ class EpubBuilder(StandaloneHTMLBuilder):
         """Create a dictionary with all metadata for the toc.ncx file
         properly escaped.
         """
-        metadata: dict[str, Any] = {}
-        metadata['uid'] = self.config.epub_uid
-        metadata['title'] = html.escape(self.config.epub_title)
-        metadata['level'] = level
-        metadata['navpoints'] = navpoints
+        metadata: dict[str, Any] = {
+            'uid': self.config.epub_uid,
+            'title': html.escape(self.config.epub_title),
+            'level': level,
+            'navpoints': navpoints,
+        }
         return metadata
 
     def build_toc(self) -> None:
@@ -757,7 +770,11 @@ class EpubBuilder(StandaloneHTMLBuilder):
 
         if self.config.epub_tocscope == 'default':
             doctree = self.env.get_and_resolve_doctree(
-                self.config.root_doc, self, prune_toctrees=False, includehidden=False
+                self.config.root_doc,
+                self,
+                tags=self.tags,
+                prune_toctrees=False,
+                includehidden=False,
             )
             refnodes = self.get_refnodes(doctree, [])
             self.toc_add_files(refnodes)
@@ -769,7 +786,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
         level = max(item['level'] for item in self.refnodes)
         level = min(level, self.config.epub_tocdepth)
         copy_asset_file(
-            path.join(self.template_dir, 'toc.ncx.jinja'),
+            self.template_dir / 'toc.ncx.jinja',
             self.outdir,
             context=self.toc_metadata(level, navpoints),
             force=True,
@@ -783,10 +800,10 @@ class EpubBuilder(StandaloneHTMLBuilder):
         """
         outname = self.config.epub_basename + '.epub'
         logger.info(__('writing %s file...'), outname)
-        epub_filename = path.join(self.outdir, outname)
+        epub_filename = self.outdir / outname
         with ZipFile(epub_filename, 'w', ZIP_DEFLATED) as epub:
-            epub.write(path.join(self.outdir, 'mimetype'), 'mimetype', ZIP_STORED)
+            epub.write(self.outdir / 'mimetype', 'mimetype', ZIP_STORED)
             for filename in ('META-INF/container.xml', 'content.opf', 'toc.ncx'):
-                epub.write(path.join(self.outdir, filename), filename, ZIP_DEFLATED)
+                epub.write(self.outdir / filename, filename, ZIP_DEFLATED)
             for filename in self.files:
-                epub.write(path.join(self.outdir, filename), filename, ZIP_DEFLATED)
+                epub.write(self.outdir / filename, filename, ZIP_DEFLATED)
