@@ -2,11 +2,12 @@
 
 from __future__ import annotations
 
-from typing import TYPE_CHECKING, TypeVar, cast
+from typing import TYPE_CHECKING, cast
 
 from docutils import nodes
 
 from sphinx import addnodes
+from sphinx.domains.std import StandardDomain
 from sphinx.environment.adapters.toctree import note_toctree
 from sphinx.environment.collectors import EnvironmentCollector
 from sphinx.locale import __
@@ -14,7 +15,7 @@ from sphinx.transforms import SphinxContentsFilter
 from sphinx.util import logging, url_re
 
 if TYPE_CHECKING:
-    from collections.abc import Sequence
+    from collections.abc import Sequence, Set
 
     from docutils.nodes import Element, Node
 
@@ -22,8 +23,6 @@ if TYPE_CHECKING:
     from sphinx.environment import BuildEnvironment
     from sphinx.util.typing import ExtensionMetadata
 
-N = TypeVar('N')
-
 logger = logging.getLogger(__name__)
 
 
@@ -42,8 +41,13 @@ class TocTreeCollector(EnvironmentCollector):
             if not fnset:
                 del env.files_to_rebuild[subfn]
 
-    def merge_other(self, app: Sphinx, env: BuildEnvironment, docnames: set[str],
-                    other: BuildEnvironment) -> None:
+    def merge_other(
+        self,
+        app: Sphinx,
+        env: BuildEnvironment,
+        docnames: Set[str],
+        other: BuildEnvironment,
+    ) -> None:
         for docname in docnames:
             env.tocs[docname] = other.tocs[docname]
             env.toc_num_entries[docname] = other.toc_num_entries[docname]
@@ -59,7 +63,7 @@ class TocTreeCollector(EnvironmentCollector):
 
     def process_doc(self, app: Sphinx, doctree: nodes.document) -> None:
         """Build a TOC from the doctree and store it in the inventory."""
-        docname = app.env.docname
+        docname = app.env.current_document.docname
         numentries = [0]  # nonlocal again...
 
         def build_toc(
@@ -83,8 +87,13 @@ class TocTreeCollector(EnvironmentCollector):
                     # make these nodes:
                     # list_item -> compact_paragraph -> reference
                     reference = nodes.reference(
-                        '', '', internal=True, refuri=docname,
-                        anchorname=anchorname, *nodetext)
+                        '',
+                        '',
+                        internal=True,
+                        refuri=docname,
+                        anchorname=anchorname,
+                        *nodetext,
+                    )
                     para = addnodes.compact_paragraph('', '', reference)
                     item: Element = nodes.list_item('', para)
                     sub_item = build_toc(sectionnode, depth + 1)
@@ -135,15 +144,23 @@ class TocTreeCollector(EnvironmentCollector):
                                 anchorname = _make_anchor_name(ids, numentries)
 
                                 reference = nodes.reference(
-                                    '', '', nodes.literal('', sig_node['_toc_name']),
-                                    internal=True, refuri=docname, anchorname=anchorname)
-                                para = addnodes.compact_paragraph('', '', reference,
-                                                                  skip_section_number=True)
+                                    '',
+                                    '',
+                                    nodes.literal('', sig_node['_toc_name']),
+                                    internal=True,
+                                    refuri=docname,
+                                    anchorname=anchorname,
+                                )
+                                para = addnodes.compact_paragraph(
+                                    '', '', reference, skip_section_number=True
+                                )
                                 entry = nodes.list_item('', para)
 
                                 # Find parent node
                                 parent = sig_node.parent
-                                while parent not in memo_parents and parent != sectionnode:
+                                while (
+                                    parent not in memo_parents and parent != sectionnode
+                                ):
                                     parent = parent.parent
                                 # Note, it may both be the limit and in memo_parents,
                                 # prefer memo_parents, so we get the nesting.
@@ -200,7 +217,7 @@ class TocTreeCollector(EnvironmentCollector):
                     _walk_toc(subnode, secnums, depth - 1, titlenode)
                     numstack.pop()
                     titlenode = None
-                elif isinstance(subnode, nodes.list_item):  # NoQA: SIM114
+                elif isinstance(subnode, nodes.list_item):
                     _walk_toc(subnode, secnums, depth, titlenode)
                     titlenode = None
                 elif isinstance(subnode, addnodes.only):
@@ -213,7 +230,7 @@ class TocTreeCollector(EnvironmentCollector):
                     if 'skip_section_number' in subnode:
                         continue
                     numstack[-1] += 1
-                    reference = cast(nodes.reference, subnode[0])
+                    reference = cast('nodes.reference', subnode[0])
                     if depth > 0:
                         number = numstack.copy()
                         secnums[reference['anchorname']] = tuple(numstack)
@@ -230,14 +247,21 @@ class TocTreeCollector(EnvironmentCollector):
         def _walk_toctree(toctreenode: addnodes.toctree, depth: int) -> None:
             if depth == 0:
                 return
-            for (_title, ref) in toctreenode['entries']:
+            for _title, ref in toctreenode['entries']:
                 if url_re.match(ref) or ref == 'self':
                     # don't mess with those
                     continue
                 if ref in assigned:
-                    logger.warning(__('%s is already assigned section numbers '
-                                      '(nested numbered toctree?)'), ref,
-                                   location=toctreenode, type='toc', subtype='secnum')
+                    logger.warning(
+                        __(
+                            '%s is already assigned section numbers '
+                            '(nested numbered toctree?)'
+                        ),
+                        ref,
+                        location=toctreenode,
+                        type='toc',
+                        subtype='secnum',
+                    )
                 elif ref in env.tocs:
                     secnums: dict[str, tuple[int, ...]] = {}
                     env.toc_secnumbers[ref] = secnums
@@ -260,7 +284,7 @@ class TocTreeCollector(EnvironmentCollector):
 
     def assign_figure_numbers(self, env: BuildEnvironment) -> list[str]:
         """Assign a figure number to each figure under a numbered toctree."""
-        generated_docnames = frozenset(env.domains['std']._virtual_doc_names)
+        generated_docnames = frozenset(env.domains.standard_domain._virtual_doc_names)
 
         rewrite_needed = []
 
@@ -270,10 +294,11 @@ class TocTreeCollector(EnvironmentCollector):
         fignum_counter: dict[str, dict[tuple[int, ...], int]] = {}
 
         def get_figtype(node: Node) -> str | None:
-            for domain in env.domains.values():
+            for domain in env.domains.sorted():
                 figtype = domain.get_enumerable_node_type(node)
-                if (domain.name == 'std'
-                        and not domain.get_numfig_title(node)):  # type: ignore[attr-defined]  # NoQA: E501
+                if isinstance(domain, StandardDomain) and not domain.get_numfig_title(
+                    node
+                ):
                     # Skip if uncaptioned node
                     continue
 
@@ -292,22 +317,27 @@ class TocTreeCollector(EnvironmentCollector):
 
             return secnum or ()
 
-        def get_next_fignumber(figtype: str, secnum: tuple[int, ...]) -> tuple[int, ...]:
+        def get_next_fignumber(
+            figtype: str, secnum: tuple[int, ...]
+        ) -> tuple[int, ...]:
             counter = fignum_counter.setdefault(figtype, {})
 
-            secnum = secnum[:env.config.numfig_secnum_depth]
+            secnum = secnum[: env.config.numfig_secnum_depth]
             counter[secnum] = counter.get(secnum, 0) + 1
             return (*secnum, counter[secnum])
 
-        def register_fignumber(docname: str, secnum: tuple[int, ...],
-                               figtype: str, fignode: Element) -> None:
+        def register_fignumber(
+            docname: str, secnum: tuple[int, ...], figtype: str, fignode: Element
+        ) -> None:
             env.toc_fignumbers.setdefault(docname, {})
             fignumbers = env.toc_fignumbers[docname].setdefault(figtype, {})
             figure_id = fignode['ids'][0]
 
             fignumbers[figure_id] = get_next_fignumber(figtype, secnum)
 
-        def _walk_doctree(docname: str, doctree: Element, secnum: tuple[int, ...]) -> None:
+        def _walk_doctree(
+            docname: str, doctree: Element, secnum: tuple[int, ...]
+        ) -> None:
             nonlocal generated_docnames
             for subnode in doctree.children:
                 if isinstance(subnode, nodes.section):
