@@ -4,28 +4,29 @@ from __future__ import annotations
 
 import re
 import unicodedata
-from typing import TYPE_CHECKING, Any, cast
+from typing import TYPE_CHECKING, cast
 
 from docutils import nodes
 from docutils.transforms import Transform, Transformer
 from docutils.transforms.parts import ContentsFilter
+from docutils.transforms.references import Footnotes
 from docutils.transforms.universal import SmartQuotes
 from docutils.utils import normalize_language_tag
 from docutils.utils.smartquotes import smartchars
 
 from sphinx import addnodes
+from sphinx.deprecation import _deprecation_warning
 from sphinx.locale import _, __
 from sphinx.util import logging
-from sphinx.util.docutils import new_document
 from sphinx.util.i18n import format_date
 from sphinx.util.nodes import apply_source_workaround, is_smartquotable
 
 if TYPE_CHECKING:
-    from collections.abc import Iterator
-    from typing import Literal
+    from collections.abc import Iterable, Iterator
+    from typing import Any, ClassVar, Literal
 
-    from docutils.nodes import Node, Text
-    from typing_extensions import TypeAlias, TypeIs
+    from docutils.nodes import Node
+    from typing_extensions import TypeIs
 
     from sphinx.application import Sphinx
     from sphinx.config import Config
@@ -33,7 +34,7 @@ if TYPE_CHECKING:
     from sphinx.environment import BuildEnvironment
     from sphinx.util.typing import ExtensionMetadata
 
-    _DEFAULT_SUBSTITUTION_NAMES: TypeAlias = Literal[
+    type _DEFAULT_SUBSTITUTION_NAMES = Literal[
         'version',
         'release',
         'today',
@@ -61,7 +62,10 @@ class SphinxTransform(Transform):
     @property
     def app(self) -> Sphinx:
         """Reference to the :class:`.Sphinx` object."""
-        return self.env.app
+        cls_module = self.__class__.__module__
+        cls_name = self.__class__.__qualname__
+        _deprecation_warning(cls_module, f'{cls_name}.app', remove=(11, 0))
+        return self.env._app
 
     @property
     def env(self) -> BuildEnvironment:
@@ -75,9 +79,7 @@ class SphinxTransform(Transform):
 
 
 class SphinxTransformer(Transformer):
-    """
-    A transformer for Sphinx.
-    """
+    """A transformer for Sphinx."""
 
     document: nodes.document
     env: BuildEnvironment | None = None
@@ -90,10 +92,12 @@ class SphinxTransformer(Transformer):
             if not hasattr(self.document.settings, 'env') and self.env:
                 self.document.settings.env = self.env
 
-            super().apply_transforms()  # type: ignore[misc]
+            super().apply_transforms()
         else:
             # wrap the target node by document node during transforming
             try:
+                from sphinx.util.docutils import new_document
+
                 document = new_document('')
                 if self.env:
                     document.settings.env = self.env
@@ -105,9 +109,7 @@ class SphinxTransformer(Transformer):
 
 
 class DefaultSubstitutions(SphinxTransform):
-    """
-    Replace some substitutions if they aren't defined in the document.
-    """
+    """Replace some substitutions if they aren't defined in the document."""
 
     # run before the default Substitutions
     default_priority = 210
@@ -119,7 +121,9 @@ class DefaultSubstitutions(SphinxTransform):
             if (name := ref['refname']) in to_handle:
                 ref.replace_self(self._handle_default_substitution(name))
 
-    def _handle_default_substitution(self, name: _DEFAULT_SUBSTITUTION_NAMES) -> nodes.Text:
+    def _handle_default_substitution(
+        self, name: _DEFAULT_SUBSTITUTION_NAMES
+    ) -> nodes.Text:
         if name == 'translation progress':
             # special handling: calculate translation progress
             return nodes.Text(_calculate_translation_progress(self.document))
@@ -127,10 +131,8 @@ class DefaultSubstitutions(SphinxTransform):
             if text := self.config.today:
                 return nodes.Text(text)
             # special handling: can also specify a strftime format
-            return nodes.Text(format_date(
-                self.config.today_fmt or _('%b %d, %Y'),
-                language=self.config.language,
-            ))
+            today_fmt = self.config.today_fmt or _('%b %d, %Y')
+            return nodes.Text(format_date(today_fmt, language=self.config.language))
         # config.version and config.release
         return nodes.Text(getattr(self.config, name))
 
@@ -149,8 +151,7 @@ def _calculate_translation_progress(document: nodes.document) -> str:
 
 
 class MoveModuleTargets(SphinxTransform):
-    """
-    Move module targets that are the first thing in a section to the section
+    """Move module targets that are the first thing in a section to the section
     title.
 
     XXX Python specific
@@ -175,17 +176,14 @@ class MoveModuleTargets(SphinxTransform):
 
 
 class HandleCodeBlocks(SphinxTransform):
-    """
-    Several code block related transformations.
-    """
+    """Several code block related transformations."""
 
     default_priority = 210
 
     def apply(self, **kwargs: Any) -> None:
         # move doctest blocks out of blockquotes
         for node in self.document.findall(nodes.block_quote):
-            if all(isinstance(child, nodes.doctest_block) for child
-                   in node.children):
+            if all(isinstance(child, nodes.doctest_block) for child in node.children):
                 node.replace_self(node.children)
         # combine successive doctest blocks
         # for node in self.document.findall(nodes.doctest_block):
@@ -200,33 +198,31 @@ class HandleCodeBlocks(SphinxTransform):
 
 
 class AutoNumbering(SphinxTransform):
-    """
-    Register IDs of tables, figures and literal_blocks to assign numbers.
-    """
+    """Register IDs of tables, figures and literal_blocks to assign numbers."""
 
     default_priority = 210
 
     def apply(self, **kwargs: Any) -> None:
-        domain: StandardDomain = self.env.domains['std']
+        domain: StandardDomain = self.env.domains.standard_domain
 
         for node in self.document.findall(nodes.Element):
-            if (domain.is_enumerable_node(node) and
-                    domain.get_numfig_title(node) is not None and
-                    node['ids'] == []):
+            if (
+                domain.is_enumerable_node(node)
+                and domain.get_numfig_title(node) is not None
+                and node['ids'] == []
+            ):
                 self.document.note_implicit_target(node)
 
 
 class SortIds(SphinxTransform):
-    """
-    Sort section IDs so that the "id[0-9]+" one comes last.
-    """
+    """Sort section IDs so that the "id[0-9]+" one comes last."""
 
     default_priority = 261
 
     def apply(self, **kwargs: Any) -> None:
         for node in self.document.findall(nodes.section):
             if len(node['ids']) > 1 and node['ids'][0].startswith('id'):
-                node['ids'] = node['ids'][1:] + [node['ids'][0]]
+                node['ids'] = [*node['ids'][1:], node['ids'][0]]
 
 
 TRANSLATABLE_NODES = {
@@ -239,30 +235,31 @@ TRANSLATABLE_NODES = {
 
 
 class ApplySourceWorkaround(SphinxTransform):
-    """
-    Update source and rawsource attributes
-    """
+    """Update source and rawsource attributes"""
 
     default_priority = 10
 
     def apply(self, **kwargs: Any) -> None:
-        for node in self.document.findall():  # type: Node
+        for node in self.document.findall():
             if isinstance(node, (nodes.TextElement, nodes.image, nodes.topic)):
                 apply_source_workaround(node)
 
 
 class AutoIndexUpgrader(SphinxTransform):
-    """
-    Detect old style (4 column based indices) and automatically upgrade to new style.
-    """
+    """Detect old style (4 column based indices) and automatically upgrade to new style."""
 
     default_priority = 210
 
     def apply(self, **kwargs: Any) -> None:
         for node in self.document.findall(addnodes.index):
             if 'entries' in node and any(len(entry) == 4 for entry in node['entries']):
-                msg = __('4 column based index found. '
-                         'It might be a bug of extensions you use: %r') % node['entries']
+                msg = (
+                    __(
+                        '4 column based index found. '
+                        'It might be a bug of extensions you use: %r'
+                    )
+                    % node['entries']
+                )
                 logger.warning(msg, location=node)
                 for i, entry in enumerate(node['entries']):
                     if len(entry) == 4:
@@ -270,9 +267,7 @@ class AutoIndexUpgrader(SphinxTransform):
 
 
 class ExtraTranslatableNodes(SphinxTransform):
-    """
-    Make nodes translatable
-    """
+    """Make nodes translatable"""
 
     default_priority = 10
 
@@ -286,31 +281,47 @@ class ExtraTranslatableNodes(SphinxTransform):
             return isinstance(node, target_nodes)
 
         for node in self.document.findall(is_translatable_node):
+            assert isinstance(node, nodes.Element)
             node['translatable'] = True
 
 
 class UnreferencedFootnotesDetector(SphinxTransform):
-    """
-    Detect unreferenced footnotes and emit warnings
-    """
+    """Detect unreferenced footnotes and emit warnings"""
 
-    default_priority = 200
+    default_priority = Footnotes.default_priority + 2
 
     def apply(self, **kwargs: Any) -> None:
         for node in self.document.footnotes:
-            if node['names'] == []:
-                # footnote having duplicated number.  It is already warned at parser.
-                pass
-            elif node['names'][0] not in self.document.footnote_refs:
-                logger.warning(__('Footnote [%s] is not referenced.'), node['names'][0],
-                               type='ref', subtype='footnote',
-                               location=node)
-
+            # note we do not warn on duplicate footnotes here
+            # (i.e. where the name has been moved to dupnames)
+            # since this is already reported by docutils
+            if not node['backrefs'] and node['names']:
+                logger.warning(
+                    __('Footnote [%s] is not referenced.'),
+                    node['names'][0] if node['names'] else node['dupnames'][0],
+                    type='ref',
+                    subtype='footnote',
+                    location=node,
+                )
+        for node in self.document.symbol_footnotes:
+            if not node['backrefs']:
+                logger.warning(
+                    __('Footnote [*] is not referenced.'),
+                    type='ref',
+                    subtype='footnote',
+                    location=node,
+                )
         for node in self.document.autofootnotes:
-            if not any(ref['auto'] == node['auto'] for ref in self.document.autofootnote_refs):
-                logger.warning(__('Footnote [#] is not referenced.'),
-                               type='ref', subtype='footnote',
-                               location=node)
+            # note we do not warn on duplicate footnotes here
+            # (i.e. where the name has been moved to dupnames)
+            # since this is already reported by docutils
+            if not node['backrefs'] and node['names']:
+                logger.warning(
+                    __('Footnote [#] is not referenced.'),
+                    type='ref',
+                    subtype='footnote',
+                    location=node,
+                )
 
 
 class DoctestTransform(SphinxTransform):
@@ -337,8 +348,7 @@ class FilterSystemMessages(SphinxTransform):
 
 
 class SphinxContentsFilter(ContentsFilter):
-    """
-    Used with BuildEnvironment.add_toc_from() to discard cross-file links
+    """Used with BuildEnvironment.add_toc_from() to discard cross-file links
     within table-of-contents link nodes.
     """
 
@@ -349,20 +359,23 @@ class SphinxContentsFilter(ContentsFilter):
 
 
 class SphinxSmartQuotes(SmartQuotes, SphinxTransform):
-    """
-    Customized SmartQuotes to avoid transform for some extra node types.
+    """Customized SmartQuotes to avoid transform for some extra node types.
 
     refs: sphinx.parsers.RSTParser
     """
 
     default_priority = 750
+    smartquotes_action: ClassVar[str] = SmartQuotes.smartquotes_action
 
     def apply(self, **kwargs: Any) -> None:
         if not self.is_available():
             return
 
         # override default settings with :confval:`smartquotes_action`
-        self.smartquotes_action = self.config.smartquotes_action
+        # TODO: TYPING: Upstream docutils should be updated so that
+        #       smartquotes_action accepts any iterable of characters
+        #       and can be overridden per-instance.
+        self.smartquotes_action = self.config.smartquotes_action  # type: ignore[misc]
 
         super().apply()
 
@@ -376,7 +389,7 @@ class SphinxSmartQuotes(SmartQuotes, SphinxTransform):
         if self.config.smartquotes is False:
             # disabled by confval smartquotes
             return False
-        if self.app.builder.name in builders:
+        if self.env._builder_cls.name in builders:
             # disabled by confval smartquotes_excludes['builders']
             return False
         if self.config.language in languages:
@@ -385,12 +398,11 @@ class SphinxSmartQuotes(SmartQuotes, SphinxTransform):
 
         # confirm selected language supports smart_quotes or not
         language = self.env.settings['language_code']
-        return any(
-            tag in smartchars.quotes
-            for tag in normalize_language_tag(language)
-        )
+        return any(tag in smartchars.quotes for tag in normalize_language_tag(language))
 
-    def get_tokens(self, txtnodes: list[Text]) -> Iterator[tuple[str, str]]:
+    def get_tokens(  # type: ignore[override]
+        self, txtnodes: Iterable[Node]
+    ) -> Iterator[tuple[Literal['literal', 'plain'], str]]:
         # A generator that yields ``(texttype, nodetext)`` tuples for a list
         # of "Text" nodes (interface to ``smartquotes.educate_tokens()``).
         for txtnode in txtnodes:
@@ -409,7 +421,7 @@ class DoctreeReadEvent(SphinxTransform):
     default_priority = 880
 
     def apply(self, **kwargs: Any) -> None:
-        self.app.emit('doctree-read', self.document)
+        self.env.events.emit('doctree-read', self.document)
 
 
 class GlossarySorter(SphinxTransform):
@@ -421,13 +433,13 @@ class GlossarySorter(SphinxTransform):
 
     def apply(self, **kwargs: Any) -> None:
         for glossary in self.document.findall(addnodes.glossary):
-            if glossary["sorted"]:
-                definition_list = cast(nodes.definition_list, glossary[0])
+            if glossary['sorted']:
+                definition_list = cast('nodes.definition_list', glossary[0])
                 definition_list[:] = sorted(
                     definition_list,
                     key=lambda item: unicodedata.normalize(
-                        'NFD',
-                        cast(nodes.term, item)[0].astext().lower()),
+                        'NFD', cast('nodes.term', item)[0].astext().lower()
+                    ),
                 )
 
 
@@ -490,7 +502,7 @@ def _reorder_index_target_nodes(start_node: nodes.target) -> None:
         first_idx = parent.index(nodes_to_reorder[0])
         last_idx = parent.index(nodes_to_reorder[-1])
         if first_idx + len(nodes_to_reorder) - 1 == last_idx:
-            parent[first_idx:last_idx + 1] = sorted(nodes_to_reorder, key=_sort_key)
+            parent[first_idx : last_idx + 1] = sorted(nodes_to_reorder, key=_sort_key)
 
 
 def _sort_key(node: nodes.Node) -> int:
