@@ -4,7 +4,7 @@ from __future__ import annotations
 
 from typing import TYPE_CHECKING, cast
 
-from docutils.writers.html4css1 import Writer
+from docutils.writers import html4css1
 
 from sphinx.util import logging
 from sphinx.writers.html5 import HTML5Translator
@@ -20,18 +20,20 @@ HTMLTranslator = HTML5Translator
 # https://www.arnebrodowski.de/blog/write-your-own-restructuredtext-writer.html
 
 
-class HTMLWriter(Writer):  # type: ignore[misc]
+class HTMLWriter(html4css1.Writer):
     # override embed-stylesheet default value to False.
     settings_default_overrides = {'embed_stylesheet': False}
 
     def __init__(self, builder: StandaloneHTMLBuilder) -> None:
         super().__init__()
         self.builder = builder
+        self._has_maths_elements: bool = False
 
     def translate(self) -> None:
         # sadly, this is mostly copied from parent class
         visitor = self.builder.create_translator(self.document, self.builder)
-        self.visitor = cast(HTML5Translator, visitor)
+        self.visitor = cast('HTML5Translator', visitor)
+        assert self.document is not None
         self.document.walkabout(visitor)
         self.output = self.visitor.astext()
         for attr in (
@@ -57,3 +59,4 @@ class HTMLWriter(Writer):  # type: ignore[misc]
         ):
             setattr(self, attr, getattr(visitor, attr, None))
         self.clean_meta = ''.join(self.visitor.meta[2:])
+        self._has_maths_elements = getattr(visitor, '_has_maths_elements', False)
