@@ -9,11 +9,18 @@ from docutils import nodes
 
 from sphinx.errors import SphinxError
 
+from tests.utils import extract_element, extract_node
+
 if TYPE_CHECKING:
+    from collections.abc import Callable
+    from pathlib import Path
+
     from sphinx.testing.util import SphinxTestApp
 
 
-def test_root_doc_not_found(tmp_path, make_app):
+def test_root_doc_not_found(
+    tmp_path: Path, make_app: Callable[..., SphinxTestApp]
+) -> None:
     (tmp_path / 'conf.py').touch()
     assert [p.name for p in tmp_path.iterdir()] == ['conf.py']
 
@@ -56,56 +63,58 @@ def test_multiple_parents_toctree(app: SphinxTestApp) -> None:
 
 @pytest.mark.usefixtures('_http_teapot')
 @pytest.mark.sphinx('dummy', testroot='images')
-def test_image_glob(app):
+def test_image_glob(app: SphinxTestApp) -> None:
     app.build(force_all=True)
 
     # index.rst
     doctree = app.env.get_doctree('index')
+    assert isinstance(doctree[0], nodes.Element)
 
-    assert isinstance(doctree[0][1], nodes.image)
-    assert doctree[0][1]['candidates'] == {'*': 'rimg.png'}
-    assert doctree[0][1]['uri'] == 'rimg.png'
+    assert isinstance(extract_node(doctree, 0, 1), nodes.image)
+    assert extract_element(doctree, 0, 1)['candidates'] == {'*': 'rimg.png'}
+    assert extract_element(doctree, 0, 1)['uri'] == 'rimg.png'
 
-    assert isinstance(doctree[0][2], nodes.figure)
-    assert isinstance(doctree[0][2][0], nodes.image)
-    assert doctree[0][2][0]['candidates'] == {'*': 'rimg.png'}
-    assert doctree[0][2][0]['uri'] == 'rimg.png'
+    assert isinstance(extract_node(doctree, 0, 2), nodes.figure)
+    assert isinstance(extract_node(doctree, 0, 2, 0), nodes.image)
+    assert extract_element(doctree, 0, 2, 0)['candidates'] == {'*': 'rimg.png'}
+    assert extract_element(doctree, 0, 2, 0)['uri'] == 'rimg.png'
 
-    assert isinstance(doctree[0][3], nodes.image)
-    assert doctree[0][3]['candidates'] == {
+    assert isinstance(extract_node(doctree, 0, 3), nodes.image)
+    assert extract_element(doctree, 0, 3)['candidates'] == {
         'application/pdf': 'img.pdf',
         'image/gif': 'img.gif',
         'image/png': 'img.png',
     }
-    assert doctree[0][3]['uri'] == 'img.*'
+    assert extract_element(doctree, 0, 3)['uri'] == 'img.*'
 
-    assert isinstance(doctree[0][4], nodes.figure)
-    assert isinstance(doctree[0][4][0], nodes.image)
-    assert doctree[0][4][0]['candidates'] == {
+    assert isinstance(extract_node(doctree, 0, 4), nodes.figure)
+    assert isinstance(extract_node(doctree, 0, 4, 0), nodes.image)
+    assert extract_element(doctree, 0, 4, 0)['candidates'] == {
         'application/pdf': 'img.pdf',
         'image/gif': 'img.gif',
         'image/png': 'img.png',
     }
-    assert doctree[0][4][0]['uri'] == 'img.*'
+    assert extract_element(doctree, 0, 4, 0)['uri'] == 'img.*'
 
     # subdir/index.rst
     doctree = app.env.get_doctree('subdir/index')
+    assert isinstance(doctree[0], nodes.Element)
 
-    assert isinstance(doctree[0][1], nodes.image)
-    assert doctree[0][1]['candidates'] == {'*': 'subdir/rimg.png'}
-    assert doctree[0][1]['uri'] == 'subdir/rimg.png'
+    assert isinstance(extract_node(doctree, 0, 1), nodes.image)
+    assert extract_element(doctree, 0, 1)['candidates'] == {'*': 'subdir/rimg.png'}
+    assert extract_element(doctree, 0, 1)['uri'] == 'subdir/rimg.png'
 
-    assert isinstance(doctree[0][2], nodes.image)
-    assert doctree[0][2]['candidates'] == {
+    assert isinstance(extract_node(doctree, 0, 2), nodes.image)
+    assert extract_element(doctree, 0, 2)['candidates'] == {
         'application/pdf': 'subdir/svgimg.pdf',
         'image/svg+xml': 'subdir/svgimg.svg',
     }
-    assert doctree[0][2]['uri'] == 'subdir/svgimg.*'
+    assert extract_element(doctree, 0, 2)['uri'] == 'subdir/svgimg.*'
 
-    assert isinstance(doctree[0][3], nodes.figure)
-    assert isinstance(doctree[0][3][0], nodes.image)
-    assert doctree[0][3][0]['candidates'] == {
+    assert isinstance(extract_node(doctree, 0, 3), nodes.figure)
+    assert isinstance(extract_node(doctree, 0, 3, 0), nodes.image)
+    assert extract_element(doctree, 0, 3, 0)['candidates'] == {
         'application/pdf': 'subdir/svgimg.pdf',
         'image/svg+xml': 'subdir/svgimg.svg',
     }
-    assert doctree[0][3][0]['uri'] == 'subdir/svgimg.*'
+    assert extract_element(doctree, 0, 3, 0)['uri'] == 'subdir/svgimg.*'
