@@ -1,5 +1,9 @@
 """Tests the reStructuredText domain."""
 
+from __future__ import annotations
+
+from typing import TYPE_CHECKING
+
 import pytest
 
 from sphinx import addnodes
@@ -15,8 +19,13 @@ from sphinx.domains.rst import parse_directive
 from sphinx.testing import restructuredtext
 from sphinx.testing.util import assert_node
 
+from tests.utils import extract_node
 
-def test_parse_directive():
+if TYPE_CHECKING:
+    from sphinx.testing.util import SphinxTestApp
+
+
+def test_parse_directive() -> None:
     s = parse_directive(' foö  ')
     assert s == ('foö', '')
 
@@ -30,8 +39,8 @@ def test_parse_directive():
     assert s == ('.. :: bar', '')
 
 
-@pytest.mark.sphinx('html', testroot='root')
-def test_rst_directive(app):
+@pytest.mark.sphinx('html', testroot='_blank')
+def test_rst_directive(app: SphinxTestApp) -> None:
     # bare
     text = '.. rst:directive:: toctree'
     doctree = restructuredtext.parse(app, text)
@@ -79,8 +88,8 @@ def test_rst_directive(app):
     )
 
 
-@pytest.mark.sphinx('html', testroot='root')
-def test_rst_directive_with_argument(app):
+@pytest.mark.sphinx('html', testroot='_blank')
+def test_rst_directive_with_argument(app: SphinxTestApp) -> None:
     text = '.. rst:directive:: .. toctree:: foo bar baz'
     doctree = restructuredtext.parse(app, text)
     assert_node(
@@ -113,8 +122,8 @@ def test_rst_directive_with_argument(app):
     )
 
 
-@pytest.mark.sphinx('html', testroot='root')
-def test_rst_directive_option(app):
+@pytest.mark.sphinx('html', testroot='_blank')
+def test_rst_directive_option(app: SphinxTestApp) -> None:
     text = '.. rst:directive:option:: foo'
     doctree = restructuredtext.parse(app, text)
     assert_node(
@@ -140,8 +149,8 @@ def test_rst_directive_option(app):
     )
 
 
-@pytest.mark.sphinx('html', testroot='root')
-def test_rst_directive_option_with_argument(app):
+@pytest.mark.sphinx('html', testroot='_blank')
+def test_rst_directive_option_with_argument(app: SphinxTestApp) -> None:
     text = '.. rst:directive:option:: foo: bar baz'
     doctree = restructuredtext.parse(app, text)
     assert_node(
@@ -176,8 +185,8 @@ def test_rst_directive_option_with_argument(app):
     )
 
 
-@pytest.mark.sphinx('html', testroot='root')
-def test_rst_directive_option_type(app):
+@pytest.mark.sphinx('html', testroot='_blank')
+def test_rst_directive_option_type(app: SphinxTestApp) -> None:
     text = '.. rst:directive:option:: foo\n   :type: directives.flags\n'
     doctree = restructuredtext.parse(app, text)
     assert_node(
@@ -215,8 +224,8 @@ def test_rst_directive_option_type(app):
     )
 
 
-@pytest.mark.sphinx('html', testroot='root')
-def test_rst_directive_and_directive_option(app):
+@pytest.mark.sphinx('html', testroot='_blank')
+def test_rst_directive_and_directive_option(app: SphinxTestApp) -> None:
     text = '.. rst:directive:: foo\n\n   .. rst:directive:option:: bar\n'
     doctree = restructuredtext.parse(app, text)
     assert_node(
@@ -233,7 +242,7 @@ def test_rst_directive_and_directive_option(app):
         ),
     )
     assert_node(
-        doctree[1][1][0],
+        extract_node(doctree, 1, 1, 0),
         entries=[
             (
                 'pair',
@@ -245,10 +254,11 @@ def test_rst_directive_and_directive_option(app):
         ],
     )
     assert_node(
-        doctree[1][1][1], ([desc_signature, desc_name, ':bar:'], [desc_content, ()])
+        extract_node(doctree, 1, 1, 1),
+        ([desc_signature, desc_name, ':bar:'], [desc_content, ()]),
     )
     assert_node(
-        doctree[1][1][1],
+        extract_node(doctree, 1, 1, 1),
         addnodes.desc,
         desctype='directive:option',
         domain='rst',
@@ -257,8 +267,8 @@ def test_rst_directive_and_directive_option(app):
     )
 
 
-@pytest.mark.sphinx('html', testroot='root')
-def test_rst_role(app):
+@pytest.mark.sphinx('html', testroot='_blank')
+def test_rst_role(app: SphinxTestApp) -> None:
     text = '.. rst:role:: ref'
     doctree = restructuredtext.parse(app, text)
     assert_node(
