@@ -1,9 +1,12 @@
 """Test math extensions."""
 
+from __future__ import annotations
+
 import re
 import shutil
 import subprocess
 import warnings
+from typing import TYPE_CHECKING
 
 import pytest
 from docutils import nodes
@@ -11,6 +14,13 @@ from docutils import nodes
 from sphinx.ext.mathjax import MATHJAX_URL
 from sphinx.testing.util import assert_node
 
+from tests.utils import extract_node
+
+if TYPE_CHECKING:
+    from sphinx.testing.util import SphinxTestApp
+
+FAKE_MATHJAX_URL = 'https://example.org/fake-mathjax.js'
+
 
 def has_binary(binary: str) -> bool:
     try:
@@ -31,7 +41,7 @@ def has_binary(binary: str) -> bool:
     testroot='ext-math-simple',
     confoverrides={'extensions': ['sphinx.ext.imgmath']},
 )
-def test_imgmath_png(app):
+def test_imgmath_png(app: SphinxTestApp) -> None:
     app.build(force_all=True)
     if "LaTeX command 'latex' cannot be run" in app.warning.getvalue():
         msg = 'LaTeX command "latex" is not available'
@@ -58,7 +68,7 @@ def test_imgmath_png(app):
     testroot='ext-math-simple',
     confoverrides={'extensions': ['sphinx.ext.imgmath'], 'imgmath_image_format': 'svg'},
 )
-def test_imgmath_svg(app):
+def test_imgmath_svg(app: SphinxTestApp) -> None:
     app.build(force_all=True)
     if "LaTeX command 'latex' cannot be run" in app.warning.getvalue():
         msg = 'LaTeX command "latex" is not available'
@@ -89,7 +99,7 @@ def test_imgmath_svg(app):
         'imgmath_embed': True,
     },
 )
-def test_imgmath_svg_embed(app):
+def test_imgmath_svg_embed(app: SphinxTestApp) -> None:
     app.build(force_all=True)
     if "LaTeX command 'latex' cannot be run" in app.warning.getvalue():
         msg = 'LaTeX command "latex" is not available'
@@ -112,14 +122,14 @@ def test_imgmath_svg_embed(app):
         'mathjax_options': {'integrity': 'sha384-0123456789'},
     },
 )
-def test_mathjax_options(app):
+def test_mathjax_options(app: SphinxTestApp) -> None:
     app.build(force_all=True)
 
     content = (app.outdir / 'index.html').read_text(encoding='utf8')
     shutil.rmtree(app.outdir)
     assert (
-        '<script async="async" integrity="sha384-0123456789" '
-        'src="https://cdn.jsdelivr.net/npm/mathjax@3/es5/tex-mml-chtml.js">'
+        '<script defer="defer" integrity="sha384-0123456789" '
+        f'src="{MATHJAX_URL}">'
         '</script>'
     ) in content
 
@@ -129,7 +139,7 @@ def test_mathjax_options(app):
     testroot='ext-math',
     confoverrides={'extensions': ['sphinx.ext.mathjax']},
 )
-def test_mathjax_align(app):
+def test_mathjax_align(app: SphinxTestApp) -> None:
     app.build(force_all=True)
 
     content = (app.outdir / 'index.html').read_text(encoding='utf8')
@@ -147,7 +157,7 @@ def test_mathjax_align(app):
     testroot='ext-math',
     confoverrides={'math_number_all': True, 'extensions': ['sphinx.ext.mathjax']},
 )
-def test_math_number_all_mathjax(app):
+def test_math_number_all_mathjax(app: SphinxTestApp) -> None:
     app.build()
 
     content = (app.outdir / 'index.html').read_text(encoding='utf8')
@@ -163,7 +173,7 @@ def test_math_number_all_mathjax(app):
     testroot='ext-math',
     confoverrides={'extensions': ['sphinx.ext.mathjax']},
 )
-def test_math_number_all_latex(app):
+def test_math_number_all_latex(app: SphinxTestApp) -> None:
     app.build()
 
     content = (app.outdir / 'projectnamenotset.tex').read_text(encoding='utf8')
@@ -204,7 +214,7 @@ def test_math_number_all_latex(app):
         'math_eqref_format': 'Eq.{number}',
     },
 )
-def test_math_eqref_format_html(app):
+def test_math_eqref_format_html(app: SphinxTestApp) -> None:
     app.build(force_all=True)
 
     content = (app.outdir / 'math.html').read_text(encoding='utf8')
@@ -224,7 +234,7 @@ def test_math_eqref_format_html(app):
         'math_eqref_format': 'Eq.{number}',
     },
 )
-def test_math_eqref_format_latex(app):
+def test_math_eqref_format_latex(app: SphinxTestApp) -> None:
     app.build(force_all=True)
 
     content = (app.outdir / 'projectnamenotset.tex').read_text(encoding='utf8')
@@ -244,7 +254,7 @@ def test_math_eqref_format_latex(app):
         'math_numfig': True,
     },
 )
-def test_mathjax_numfig_html(app):
+def test_mathjax_numfig_html(app: SphinxTestApp) -> None:
     app.build(force_all=True)
 
     content = (app.outdir / 'math.html').read_text(encoding='utf8')
@@ -271,7 +281,7 @@ def test_mathjax_numfig_html(app):
         'math_numsep': '-',
     },
 )
-def test_mathjax_numsep_html(app):
+def test_mathjax_numsep_html(app: SphinxTestApp) -> None:
     app.build(force_all=True)
 
     content = (app.outdir / 'math.html').read_text(encoding='utf8')
@@ -299,7 +309,7 @@ def test_mathjax_numsep_html(app):
         'math_numfig': True,
     },
 )
-def test_imgmath_numfig_html(app):
+def test_imgmath_numfig_html(app: SphinxTestApp) -> None:
     app.build(force_all=True)
 
     content = (app.outdir / 'page.html').read_text(encoding='utf8')
@@ -314,11 +324,10 @@ def test_imgmath_numfig_html(app):
 
 
 @pytest.mark.sphinx('dummy', testroot='ext-math-compat')
-def test_math_compat(app):
+def test_math_compat(app: SphinxTestApp) -> None:
     with warnings.catch_warnings(record=True):
         app.build(force_all=True)
-        doctree = app.env.get_and_resolve_doctree('index', app.builder)
-
+        doctree = app.env.get_and_resolve_doctree('index', app.builder, tags=app.tags)
         assert_node(
             doctree,
             [
@@ -332,7 +341,7 @@ def test_math_compat(app):
             ],
         )
         assert_node(
-            doctree[0][1][1],
+            extract_node(doctree, 0, 1, 1),
             (
                 'Inline: ',
                 [nodes.math, 'E=mc^2'],
@@ -341,7 +350,7 @@ def test_math_compat(app):
             ),
         )
         assert_node(
-            doctree[0][2],
+            extract_node(doctree, 0, 2),
             (
                 [nodes.title, 'block'],
                 [nodes.math_block, 'a^2+b^2=c^2\n\n'],
@@ -358,10 +367,10 @@ def test_math_compat(app):
     testroot='ext-math',
     confoverrides={
         'extensions': ['sphinx.ext.mathjax'],
-        'mathjax3_config': {'extensions': ['tex2jax.js']},
+        'mathjax4_config': {'extensions': ['tex2jax.js']},
     },
 )
-def test_mathjax3_config(app):
+def test_mathjax4_config(app: SphinxTestApp) -> None:
     app.build(force_all=True)
 
     content = (app.outdir / 'index.html').read_text(encoding='utf8')
@@ -375,14 +384,52 @@ def test_mathjax3_config(app):
     testroot='ext-math',
     confoverrides={
         'extensions': ['sphinx.ext.mathjax'],
-        'mathjax2_config': {'extensions': ['tex2jax.js']},
+        'mathjax3_config': {'extensions': ['tex2jax.js']},
+        'mathjax_path': FAKE_MATHJAX_URL,
     },
 )
-def test_mathjax2_config(app):
+def test_mathjax3_config(app: SphinxTestApp) -> None:
     app.build(force_all=True)
 
     content = (app.outdir / 'index.html').read_text(encoding='utf8')
-    assert '<script async="async" src="%s">' % MATHJAX_URL in content
+    assert FAKE_MATHJAX_URL in content
+    assert f'<script defer="defer" src="{FAKE_MATHJAX_URL}">' in content
+    assert '<script>window.MathJax = {"extensions": ["tex2jax.js"]}</script>' in content
+
+
+@pytest.mark.sphinx(
+    'html',
+    testroot='ext-math',
+    confoverrides={
+        'extensions': ['sphinx.ext.mathjax'],
+        'mathjax_config_path': '_static/custom_mathjax_config.js',
+    },
+)
+def test_mathjax_config_path_config(app: SphinxTestApp) -> None:
+    app.build(force_all=True)
+
+    content = (app.outdir / 'index.html').read_text(encoding='utf8')
+    assert MATHJAX_URL in content
+    assert f'<script defer="defer" src="{MATHJAX_URL}">' in content
+    assert (
+        '<script>window.MathJax = {"extensions": ["tex2jax.js"]}\n</script>'
+    ) in content
+
+
+@pytest.mark.sphinx(
+    'html',
+    testroot='ext-math',
+    confoverrides={
+        'extensions': ['sphinx.ext.mathjax'],
+        'mathjax2_config': {'extensions': ['tex2jax.js']},
+        'mathjax_path': FAKE_MATHJAX_URL,
+    },
+)
+def test_mathjax2_config(app: SphinxTestApp) -> None:
+    app.build(force_all=True)
+
+    content = (app.outdir / 'index.html').read_text(encoding='utf8')
+    assert f'<script async="async" src="{FAKE_MATHJAX_URL}">' in content
     assert (
         '<script type="text/x-mathjax-config">'
         'MathJax.Hub.Config({"extensions": ["tex2jax.js"]})'
@@ -396,10 +443,10 @@ def test_mathjax2_config(app):
     confoverrides={
         'extensions': ['sphinx.ext.mathjax'],
         'mathjax_options': {'async': 'async'},
-        'mathjax3_config': {'extensions': ['tex2jax.js']},
+        'mathjax4_config': {'extensions': ['tex2jax.js']},
     },
 )
-def test_mathjax_options_async_for_mathjax3(app):
+def test_mathjax_options_async_for_mathjax4(app: SphinxTestApp) -> None:
     app.build(force_all=True)
 
     content = (app.outdir / 'index.html').read_text(encoding='utf8')
@@ -416,7 +463,7 @@ def test_mathjax_options_async_for_mathjax3(app):
         'mathjax2_config': {'extensions': ['tex2jax.js']},
     },
 )
-def test_mathjax_options_defer_for_mathjax2(app):
+def test_mathjax_options_defer_for_mathjax2(app: SphinxTestApp) -> None:
     app.build(force_all=True)
 
     content = (app.outdir / 'index.html').read_text(encoding='utf8')
@@ -431,11 +478,11 @@ def test_mathjax_options_defer_for_mathjax2(app):
         'mathjax_path': 'MathJax.js',
     },
 )
-def test_mathjax_path(app):
+def test_mathjax_path(app: SphinxTestApp) -> None:
     app.build(force_all=True)
 
     content = (app.outdir / 'index.html').read_text(encoding='utf8')
-    assert '<script async="async" src="_static/MathJax.js"></script>' in content
+    assert '<script defer="defer" src="_static/MathJax.js"></script>' in content
 
 
 @pytest.mark.sphinx(
@@ -446,12 +493,12 @@ def test_mathjax_path(app):
         'mathjax_path': 'MathJax.js?config=scipy-mathjax',
     },
 )
-def test_mathjax_path_config(app):
+def test_mathjax_path_config(app: SphinxTestApp) -> None:
     app.build(force_all=True)
 
     content = (app.outdir / 'index.html').read_text(encoding='utf8')
     assert (
-        '<script async="async" src="_static/MathJax.js?config=scipy-mathjax"></script>'
+        '<script defer="defer" src="_static/MathJax.js?config=scipy-mathjax"></script>'
     ) in content
 
 
@@ -460,7 +507,7 @@ def test_mathjax_path_config(app):
     testroot='ext-math',
     confoverrides={'extensions': ['sphinx.ext.mathjax']},
 )
-def test_mathjax_is_installed_only_if_document_having_math(app):
+def test_mathjax_is_installed_only_if_document_having_math(app: SphinxTestApp) -> None:
     app.build(force_all=True)
 
     content = (app.outdir / 'index.html').read_text(encoding='utf8')
@@ -475,11 +522,12 @@ def test_mathjax_is_installed_only_if_document_having_math(app):
     testroot='basic',
     confoverrides={'extensions': ['sphinx.ext.mathjax']},
 )
-def test_mathjax_is_not_installed_if_no_equations(app):
+def test_mathjax_is_not_installed_if_no_equations(app: SphinxTestApp) -> None:
     app.build(force_all=True)
 
     content = (app.outdir / 'index.html').read_text(encoding='utf8')
     assert 'MathJax.js' not in content
+    assert MATHJAX_URL not in content
 
 
 @pytest.mark.sphinx(
@@ -487,7 +535,7 @@ def test_mathjax_is_not_installed_if_no_equations(app):
     testroot='ext-math',
     confoverrides={'extensions': ['sphinx.ext.mathjax']},
 )
-def test_mathjax_is_installed_if_no_equations_when_forced(app):
+def test_mathjax_is_installed_if_no_equations_when_forced(app: SphinxTestApp) -> None:
     app.set_html_assets_policy('always')
     app.build(force_all=True)
 
@@ -503,7 +551,9 @@ def test_mathjax_is_installed_if_no_equations_when_forced(app):
     testroot='ext-math-include',
     confoverrides={'extensions': ['sphinx.ext.mathjax']},
 )
-def test_mathjax_is_installed_if_included_file_has_equations(app):
+def test_mathjax_is_installed_if_included_file_has_equations(
+    app: SphinxTestApp,
+) -> None:
     app.build(force_all=True)
 
     # no real equations at the rst level, but includes "included"
@@ -523,7 +573,9 @@ def test_mathjax_is_installed_if_included_file_has_equations(app):
     testroot='ext-math',
     confoverrides={'extensions': ['sphinx.ext.mathjax']},
 )
-def test_mathjax_is_installed_only_if_document_having_math_singlehtml(app):
+def test_mathjax_is_installed_only_if_document_having_math_singlehtml(
+    app: SphinxTestApp,
+) -> None:
     app.build(force_all=True)
 
     content = (app.outdir / 'index.html').read_text(encoding='utf8')
@@ -535,11 +587,14 @@ def test_mathjax_is_installed_only_if_document_having_math_singlehtml(app):
     testroot='basic',
     confoverrides={'extensions': ['sphinx.ext.mathjax']},
 )
-def test_mathjax_is_not_installed_if_no_equations_singlehtml(app):
+def test_mathjax_is_not_installed_if_no_equations_singlehtml(
+    app: SphinxTestApp,
+) -> None:
     app.build(force_all=True)
 
     content = (app.outdir / 'index.html').read_text(encoding='utf8')
     assert 'MathJax.js' not in content
+    assert MATHJAX_URL not in content
 
 
 @pytest.mark.sphinx(
@@ -547,8 +602,26 @@ def test_mathjax_is_not_installed_if_no_equations_singlehtml(app):
     testroot='ext-math-include',
     confoverrides={'extensions': ['sphinx.ext.mathjax']},
 )
-def test_mathjax_is_installed_if_included_file_has_equations_singlehtml(app):
+def test_mathjax_is_installed_if_included_file_has_equations_singlehtml(
+    app: SphinxTestApp,
+) -> None:
     app.build(force_all=True)
 
     content = (app.outdir / 'index.html').read_text(encoding='utf8')
     assert MATHJAX_URL in content
+
+
+@pytest.mark.sphinx(
+    'html',
+    testroot='ext-math-duplicate-label',
+    confoverrides={'extensions': ['sphinx.ext.mathjax'], 'show_warning_types': True},
+)
+def test_duplicate_equation_label_warning_type(app: SphinxTestApp) -> None:
+    """Test that duplicate equation labels emit warnings with type ref.equation."""
+    app.build(force_all=True)
+
+    from sphinx._cli.util.errors import strip_escape_sequences
+
+    warnings = strip_escape_sequences(app.warning.getvalue())
+    assert 'WARNING: duplicate label of equation duplicated' in warnings
+    assert '[ref.equation]' in warnings
