@@ -2,7 +2,6 @@
 
 from __future__ import annotations
 
-import codecs
 import os
 from contextlib import contextmanager
 from pathlib import Path
@@ -18,16 +17,18 @@ from sphinx.util.console import colorize
 from sphinx.util.logging import is_suppressed_warning, prefixed_warnings
 from sphinx.util.parallel import ParallelTasks
 
-from tests.utils import TESTS_ROOT
+from tests.utils import TEST_ROOTS_DIR
 
 if TYPE_CHECKING:
+    import io
+    from collections.abc import Iterator
+
     from sphinx.testing.util import SphinxTestApp
 
 
 @pytest.mark.sphinx('html', testroot='root')
 def test_info_and_warning(app: SphinxTestApp) -> None:
-    app.verbosity = 2
-    logging.setup(app, app.status, app.warning)
+    logging.setup(app, app.status, app.warning, verbosity=2)
     logger = logging.getLogger(__name__)
 
     logger.debug('message1')
@@ -61,8 +62,7 @@ def test_Exception(app: SphinxTestApp) -> None:
 @pytest.mark.sphinx('html', testroot='root')
 def test_verbosity_filter(app: SphinxTestApp) -> None:
     # verbosity = 0: INFO
-    app.verbosity = 0
-    logging.setup(app, app.status, app.warning)
+    logging.setup(app, app.status, app.warning, verbosity=0)
     logger = logging.getLogger(__name__)
 
     logger.info('message1')
@@ -75,8 +75,7 @@ def test_verbosity_filter(app: SphinxTestApp) -> None:
     assert 'message4' not in app.status.getvalue()
 
     # verbosity = 1: VERBOSE
-    app.verbosity = 1
-    logging.setup(app, app.status, app.warning)
+    logging.setup(app, app.status, app.warning, verbosity=1)
     logger = logging.getLogger(__name__)
 
     logger.info('message1')
@@ -89,8 +88,7 @@ def test_verbosity_filter(app: SphinxTestApp) -> None:
     assert 'message4' not in app.status.getvalue()
 
     # verbosity = 2: DEBUG
-    app.verbosity = 2
-    logging.setup(app, app.status, app.warning)
+    logging.setup(app, app.status, app.warning, verbosity=2)
     logger = logging.getLogger(__name__)
 
     logger.info('message1')
@@ -128,16 +126,16 @@ def test_once_warning_log(app: SphinxTestApp) -> None:
     assert 'WARNING: message: 1\nWARNING: message: 2\n' in warnings
 
 
-def test_is_suppressed_warning():
+def test_is_suppressed_warning() -> None:
     suppress_warnings = ['ref', 'files.*', 'rest.duplicated_labels']
 
-    assert is_suppressed_warning(None, None, suppress_warnings) is False
-    assert is_suppressed_warning('ref', None, suppress_warnings) is True
+    assert is_suppressed_warning(None, None, suppress_warnings) is False  # type: ignore[arg-type]
+    assert is_suppressed_warning('ref', None, suppress_warnings) is True  # type: ignore[arg-type]
     assert is_suppressed_warning('ref', 'numref', suppress_warnings) is True
     assert is_suppressed_warning('ref', 'option', suppress_warnings) is True
     assert is_suppressed_warning('files', 'image', suppress_warnings) is True
     assert is_suppressed_warning('files', 'stylesheet', suppress_warnings) is True
-    assert is_suppressed_warning('rest', None, suppress_warnings) is False
+    assert is_suppressed_warning('rest', None, suppress_warnings) is False  # type: ignore[arg-type]
     assert is_suppressed_warning('rest', 'syntax', suppress_warnings) is False
     assert is_suppressed_warning('rest', 'duplicated_labels', suppress_warnings) is True
 
@@ -285,7 +283,7 @@ def test_pending_warnings(app: SphinxTestApp) -> None:
 
 
 @contextmanager
-def force_colors():
+def force_colors() -> Iterator[None]:
     forcecolor = os.environ.get('FORCE_COLOR', None)
 
     try:
@@ -298,10 +296,10 @@ def force_colors():
             os.environ['FORCE_COLOR'] = forcecolor
 
 
-def test_log_no_ansi_colors(tmp_path):
+def test_log_no_ansi_colors(tmp_path: Path) -> None:
     with force_colors():
         wfile = tmp_path / 'warnings.txt'
-        srcdir = TESTS_ROOT / 'roots' / 'test-nitpicky-warnings'
+        srcdir = TEST_ROOTS_DIR / 'test-nitpicky-warnings'
         argv = list(map(str, ['-b', 'html', srcdir, tmp_path, '-n', '-w', wfile]))
         retcode = build_main(argv)
         assert retcode == 0
@@ -312,8 +310,7 @@ def test_log_no_ansi_colors(tmp_path):
 
 @pytest.mark.sphinx('html', testroot='root')
 def test_colored_logs(app: SphinxTestApp) -> None:
-    app.verbosity = 2
-    logging.setup(app, app.status, app.warning)
+    logging.setup(app, app.status, app.warning, verbosity=2)
     logger = logging.getLogger(__name__)
 
     # default colors
@@ -346,7 +343,7 @@ def test_logging_in_ParallelTasks(app: SphinxTestApp) -> None:
     logging.setup(app, app.status, app.warning)
     logger = logging.getLogger(__name__)
 
-    def child_process():
+    def child_process() -> None:
         logger.info('message1')
         logger.warning('message2', location='index')
 
@@ -358,12 +355,15 @@ def test_logging_in_ParallelTasks(app: SphinxTestApp) -> None:
 
 
 @pytest.mark.sphinx('html', testroot='root')
-def test_output_with_unencodable_char(app):
-    class StreamWriter(codecs.StreamWriter):
-        def write(self, object):
+def test_output_with_unencodable_char(app: SphinxTestApp) -> None:
+    class StreamWriter:
+        def __init__(self, stream: io.StringIO) -> None:
+            self.stream = stream
+
+        def write(self, object: str) -> None:
             self.stream.write(object.encode('cp1252').decode('cp1252'))
 
-    logging.setup(app, StreamWriter(app.status), app.warning)
+    logging.setup(app, StreamWriter(app.status), app.warning, verbosity=0)  # type: ignore[arg-type]
     logger = logging.getLogger(__name__)
 
     # info with UnicodeEncodeError
@@ -393,7 +393,7 @@ def test_prefixed_warnings(app: SphinxTestApp) -> None:
     assert 'WARNING: message5' in app.warning.getvalue()
 
 
-def test_get_node_location_abspath():
+def test_get_node_location_abspath() -> None:
     # Ensure that node locations are reported as an absolute path,
     # even if the source attribute is a relative path.
 
