@@ -1,7 +1,9 @@
+from __future__ import annotations
+
 from sphinx.util._uri import encode_uri
 
 
-def test_encode_uri():
+def test_encode_uri() -> None:
     expected = (
         'https://ru.wikipedia.org/wiki/%D0%A1%D0%B8%D1%81%D1%82%D0%B5%D0%BC%D0%B0_'
         '%D1%83%D0%BF%D1%80%D0%B0%D0%B2%D0%BB%D0%B5%D0%BD%D0%B8%D1%8F_'
